%%% ----------------------------------------------------------
%%%
%%% The contents of this file are subject to the Erlang Public License,
%%% Version 1.0, (the "License"); you may not use this file except in
%%% compliance with the License. You may obtain a copy of the License at
%%% http://www.erlang.org/license/EPL1_0.txt
%%%
%%% Software distributed under the License is distributed on an "AS IS"
%%% basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
%%% the License for the specific language governing rights and limitations
%%% under the License.
%%%
%%% The Original Code is ccviewer-1.1.
%%%
%%% The Initial Developer of the Original Code is Ericsson Telecom
%%% AB. Portions created by Ericsson are Copyright (C), 1998, Ericsson
%%% Telecom AB. All Rights Reserved.
%%%
%%% Contributor(s): ______________________________________.

%%%----------------------------------------------------------------------
%%% #0.    BASIC INFORMATION
%%%----------------------------------------------------------------------
%%% %CCaseFile:	ccv_cvs.erl %
%%% Author       : Ulf Wiger <ulf.wiger@ericsson.com>
%%% Description  : ClearCase file server for CCviewer.
%%% 
%%% Modules used : 
%%%----------------------------------------------------------------------
-module(ccv_cvs).
-id('55/190 55-LXA 119 34 Ux').
-vsn('/main/8').
-date('2005-02-08').
-author('etxuwig').


%%% ----------------------------------------------------------
%%% #1.    REVISION LOG
%%% ----------------------------------------------------------
%%% Rev        Date       Name        What
%%% --------   --------   --------    ------------------------
%%% <Brnch/n>  <98xxxx>   <userid>    <text>
%%% ----------------------------------------------------------
%%% 
%%% #2.    EXPORT LISTS
%%% ----------------------------------------------------------
%%% #2.1   EXPORTED INTERFACE FUNCTIONS
%%% ----------------------------------------------------------

-export([
	 bootstrap/0,
	 initialize/0,
	 cleanup/0
	]).

-export([internal_filename/2,
	 public_filename/1,
	 is_direct/2,
	 read_file/1,
	 open/2,
	 read_file_info/1,
	 read_link/1,
	 list_dir/1,
	 diff/2]).


%%%----------------------------------------------------------------------
%%% #2.2   EXPORTED INTERNAL FUNCTIONS
%%%----------------------------------------------------------------------


-include("ccviewer.hrl").
-include("ccv_locale.hrl").
-include_lib("kernel/include/file.hrl").

-define(TYPE_TAG, ?MODULE).

-define(MYCVS, "/home/etxuwig/work/cvs/").
-define(MYCVSROOT, "/home/etxuwig/work/cvsroot").
%%%----------------------------------------------------------------------
%%% #3.    CODE
%%%----------------------------------------------------------------------
%%% #3.1   CODE FOR EXPORTED INTERFACE FUNCTIONS
%%%----------------------------------------------------------------------

%%% =============================================================
%%% API Functions
%%% =============================================================

bootstrap() ->
    ok.

initialize() ->
    ok.

cleanup() ->
    ok.




%% read_file(Filename, CS) -> {ok, Bin} | Error
%%
read_file(#ccv_file{type = ?TYPE_TAG,
		    internal = {CVSRoot, Label, Name}}) ->
    TempDir = export_module(CVSRoot, Label, Name),
    Basename = filename:basename(Name),
    Fullname = filename:join(TempDir, Basename),
    Res = file:read_file(Fullname),
    ccv_fs:remove_dir(TempDir, [recurse]),
    Res.

internal_filename(?MYCVS ++ Rest = Fname, CS) ->
    CVSRoot = ?MYCVSROOT,
    Filename = filename:join(CVSRoot, Rest),
    VsnFileName = Filename ++ ",v",
    case file:read_link_info(VsnFileName) of
	{ok, _FI} ->
	    Vsn = ccv_cvs_lib:version(VsnFileName, CS),
	    #ccv_file{type = ?TYPE_TAG,
		      internal = {CVSRoot,CS,Rest},
		      vsn = Vsn,
		      public = Fname};
	_Error ->
	    %% try with vsn = undefined -- may not work
	    #ccv_file{type = ?TYPE_TAG,
		      internal = {CVSRoot,CS,Rest},
		      public = Fname}
    end.
    


public_filename(#ccv_file{type = ?TYPE_TAG,
			  public = Public}) ->
    Public.


%% ... Not sure about this. I have not tested with CVS enough to know that
%% this is what would in fact be passed to the is_internal/2 function.
is_direct({?MYCVSROOT, CS, Rest} = Internal, CS) ->
    #ccv_file{type = ?TYPE_TAG,
	      cs = CS,
	      internal = Internal,
	      public = ?MYCVS ++ Rest};
is_direct(_, _) ->
    false.


read_file_info(#ccv_file{type = ?TYPE_TAG,
			 internal = {CVSRoot, Label, Name}}) ->
    %% this is just a temporary hack -- not a correct implementation
    Filename = filename:join(CVSRoot, Name),
    case file:read_link_info(Filename) of
	{error, enoent} ->
	    %% should be a version-controlled file
	    VsnFileName = Filename ++ ",v",
	    case file:read_link_info(VsnFileName) of
		{ok, FI} ->
		    Date = ccv_cvs_lib:modified_date(VsnFileName, Label),
		    {ok, FI#file_info{mtime = Date}};
		{error, Reason} ->
		    exit({Reason, VsnFileName})
	    end;
	{ok, #file_info{type = directory}} = Result ->
	    Result
    end.

read_link(#ccv_file{type = ?TYPE_TAG}) ->
    %% Can we really have symlinks inside the CVS repository??
    {error, enotsup}.
		    

open(#ccv_file{type = ?TYPE_TAG,
	       internal = {CVSRoot, _Label, Name}}, Args) ->
        %% this is just a temporary hack -- not a correct implementation
    Filename = filename:join(CVSRoot, Name),
    case file:open(Filename, Args) of
	{error, enoent} ->
	    %% should be a version-controlled file
	    VsnFileName = Filename ++ ",v",
	    case file:open(VsnFileName, Args) of
		{ok, Fd} ->
		    {ok, Fd};
		{error, Reason} ->
		    exit({Reason, VsnFileName})
	    end;
	Result ->
	    Result
    end.



list_dir(#ccv_file{type = ?TYPE_TAG,
		   internal = {Root, Tag, Dir}}) ->
    TempDir = export_module(Root, Tag, Dir),
    Res = file:list_dir(TempDir),
    io:format("list_dir(Filename) -> ~p~n", [Res]),
    ccv_fs:remove_dir(TempDir, [recurse]),
    Res.


%% this is copied from the ClearCase implementation - won't work with CVS
diff(#ccv_file{type = ?TYPE_TAG,
	       internal = ["/view/", _, _] = Old},
     #ccv_file{type = ?TYPE_TAG,
	       internal = ["/view/", _, _] = New}) ->
    ccv_lib:os_cmd(["diff ", Old, " ", New]).

%%%----------------------------------------------------------------------
%%% #3.2   CODE FOR EXPORTED INTERNAL FUNCTIONS
%%%----------------------------------------------------------------------



%%%----------------------------------------------------------------------
%%% #3.3   CODE FOR INTERNAL FUNCTIONS
%%%----------------------------------------------------------------------

export_module(CVSROOT, Tag, File) ->
    Dir = filename:join("/tmp", unique_name()),
    file:make_dir(Dir),
    Cmd = [cvs(), " -t -d ", CVSROOT, " export -r ", Tag, 
	   " -d ", Dir, " ", File],
    Result = ccv_lib:os_cmd(Cmd),
    io:format("~s~n", [Result]),
    Dir.


cvs() ->
    "/EDUP/Cvs/1.7/bin/cvs".

unique_name() ->
    {Ms, S, Us} = erlang:now(),
    "ccv_cvs-" ++ integer_to_list(Ms) ++
	"-" ++ integer_to_list(S) ++
	"-" ++ integer_to_list(Us).


%%% =============================================================
%%% Environment Setup
%%% =============================================================


%%% =============================================================
%%% Environment cleanup
%%% =============================================================
