%%% #0.    BASIC INFORMATION
%%% ----------------------------------------------------------
%%% %CCaseFile:	ccviewer.hrl %
%%% Author:          Ulf Wiger          <ulf.wiger@ericsson.com>
%%% Description:     Record definitions for CCviewer
%%%
%%% ----------------------------------------------------------
-hrl_id('18/190 55-LXA 119 34 Ux').
-hrl_vsn('/main/31').
-hrl_date('2005-02-08').
-hrl_author('etxuwig').

%% ccviewer.hrl

%%% ----------------------------------------------------------
%%% #1.    REVISION LOG
%%% ----------------------------------------------------------
%%% Rev        Date     Name       What
%%% --------   ------   --------   ---------------------------
%%% <Brnch/4>  000105   etxuwig    Added email to profile
%%% ----------------------------------------------------------
%%%
%%% ----------------------------------------------------------
%%% #2.    CODE
%%% #---------------------------------------------------------
%%% #2.1   DEFINITION OF CONSTANTS
%%% #---------------------------------------------------------
%%% ----------------------------------------------------------
%%% #           CONSTANT_NAME
%%% Description: 
%%% ----------------------------------------------------------

-ifndef(dbg).
-define(dbg(Fmt, Args), no_debug).
%-define(dbg(Fmt, Args), io:format("~p/~p - " ++ Fmt, [?MODULE, ?LINE|Args])).
-endif.


-define(macro_arity_ext, "d").	%% used by the xrefs database functions
-define(macro_arity_int, d).	%% used by the xrefs database functions
-define(record_arity_ext, "r").	%% used by the xrefs database functions
-define(record_arity_int, r).	%% used by the xrefs database functions
-define(attr_arity_ext, "a").	%% used by the xrefs database functions
-define(attr_arity_int, a).	%% used by the xrefs database functions

%%% #---------------------------------------------------------
%%% #2.2   DEFINITION OF RECORDS
%%% #---------------------------------------------------------

%%% ----------------------------------------------------------
%%% #           ccv_req
%%% Description: Request record
%%% ----------------------------------------------------------
-record(ccv_req, 
	{cs,			% the current CS
	 curFile="",		% the current file "" if no current file
	 internal_fname = "",	% The fully qualified filename
	 curDir="",		% the current directory
	 files="",		% File Spec field
	 set_origin = false,	% true if curFile is not in the cache
	 pattern="",		% Grep Pattern
	 formatted = false,	% true if we've modified the displayed file
	 user = "guest",
	 remote_user = [],	% [] | {User,RemotePeer} if remote user
	 address,
	 page_url = "",		% Redirect location
	 history_index,
	 previous,
	 redirect = false,
	 use_cookies = false}).

%%% ----------------------------------------------------------
%%% #           ccv_page
%%% Description: Page descriptor
%%% ----------------------------------------------------------
-record(ccv_page, 
	{data = "",
	 title = "",
	 styles = ""}).


%%% ----------------------------------------------------------
%%% #           ccv_public
%%% Description: Table of publicly available documents
%%% ----------------------------------------------------------
-record(ccv_public,
	{key,
	 description = "",
	 filename,
	 cs}).

%%% ----------------------------------------------------------
%%% #           ccv_source
%%% Description: Source code cache
%%% ----------------------------------------------------------
-record(ccv_source, {key, value}).


%%% ----------------------------------------------------------
%%% #           ccv_vfs
%%% Description: Virtual File System
%%% ----------------------------------------------------------
-record(ccv_vfs, {key,
		  mod,
		  prefix = [],
		  acl = [],
		  cs = same}).

%%% ----------------------------------------------------------
%%% #           ccv_env
%%% Description: Source code cache
%%% ----------------------------------------------------------
-record(ccv_env, {key, value}).


%%% ----------------------------------------------------------
%%% #           ccv_views
%%% Description: Table of active clearcase views
%%%              This is a persistent table
%%% ----------------------------------------------------------
-record(ccv_views,  {cs,	% Config Spec name (string())
		     csf,	% Config Spec filename (string())
		     vname,	% View Name
		     vfname,	% View filename
		     otp,	% OTP Root Dir
		     cache_status,
		     xrefs_status,
		     view_status,	% faulty|stopped|undefined
		     visible = false,	% true | false
		     type = normal}).   % normal | view_only | #derived_cs{}
%% update ccv_files: if view_only, no hypertext linking (no pretty-print?)
%% update ccv_xrefs: if derived cs, and no cache, do not store xrefs, etc.


%%% ----------------------------------------------------------
%%% #           ccv_view_flags
%%% Description: Reflects the dynamic nature of views
%%%              This is a RAM-only table.
%%% ----------------------------------------------------------
-record(ccv_view_flags, {cs,			% config spec name
			 use_cache = true,
			 use_xrefs = false,
			 use_function_links = false
			}).


-record(ccv_derived_cs, {use_cs,
			 cache_updates = false}).
			 

%%% ----------------------------------------------------------
%%% #           ccv_session
%%% Description: Active sessions
%%% ----------------------------------------------------------
-record(ccv_session, {key, 
		      history,
		      next_index = 1,
		      timer}).


%%% ----------------------------------------------------------
%%% #           ccv_aliases
%%% Description: Directory aliases
%%% ----------------------------------------------------------
-record(ccv_aliases, {key, value}).

%%% ----------------------------------------------------------
%%% #           ccv_xrefs
%%% Description: Function call cross references
%%% ----------------------------------------------------------
-record(ccv_xrefs, {key,	 % {Mod, Fun, Args, FromFile, FromLine}
		    fromFun,	 % name of calling function
		    fromArity}). % arity of calling function

-record(ccv_exports, {key,		% {CS : atom(), Module : atom()}
		      exports = []}).	% [{Function, Arity}]

-record(ccv_includes, {key,		% {CS : atom(), Module : atom()}
		      includes = []}).	% [{ShortName, Fullname | not_found}]

-record(ccv_companion_data, {key,		% logkey()
			     data = <<>>}).	% binary()

-record(ccv_xrefs_log, {key,		% logkey() : {Filename, CS}
			module,		% module name (atom())
			modified}).	% Modification time

-record(ccv_file_vsn, {key, config_specs = []}).
-record(ccv_cs_file,  {key, vsn}).

%%% ----------------------------------------------------------
%%% #           ccv_object
%%% Description: Language object cache
%%% ----------------------------------------------------------
-record(ccv_object, {key,	% {CS, Name, record | define, File}
		     line}).	% line number in File

%%% ----------------------------------------------------------
%%% #           ccv_rpc
%%% Description: Remote host information
%%% ----------------------------------------------------------
-record(ccv_rpc, {identity,	% {HostName, Port}
		  my_username,
		  my_password,
		  his_username,
		  his_password,
		  cs_map = []}).	% [{MyCS, HisCS}]

%%% ----------------------------------------------------------
%%% #           ccv_subscriptions
%%% Description: Subscription information
%%% ----------------------------------------------------------
-record(ccv_subscriptions, {key, vsn}).


%%% ----------------------------------------------------------
%%% #           ccv_user
%%% Description: User accounts database
%%% ----------------------------------------------------------
-record(ccv_user, {user, 
		   password, 
		   admin_rights = false,
		   groups = [],
		   addresses = [],
		   stored_pages = [],
		   profile}).

%%% ----------------------------------------------------------
%%% #           ccv_stats
%%% Description: Statistics database
%%% ----------------------------------------------------------
-record(ccv_stats, {key, value}).

%%% ----------------------------------------------------------
%%% #           ccv_colours
%%% Description: Colour preferences
%%% ----------------------------------------------------------
-record(ccv_colours, {background= "ivory",
		      function	= "green",
		      comment	= "maroon",
		      lineno	= "silver",
		      link	= "navy"}).

%%% ----------------------------------------------------------
%%% #           ccv_profile
%%% Description: User profile
%%% ----------------------------------------------------------

-record(ccv_profile, {name = "",
		      title = "",
		      company = "",
		      phone = "",
		      email = "",
		      fontsize = "10pt",
		      hide_comments = false,
		      hide_debugs = false,
		      reduce_whitespace = false,
		      parse = true,
		      cs = auto,
		      colours = #ccv_colours{}}).
